package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.ui.SceneAwareController;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Testbed;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.connection.*;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.IOUtils;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import com.github.rjeschke.txtmark.Run;
import com.sun.javafx.PlatformUtil;
import com.sun.javafx.webkit.WebConsoleListener;
import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import javafx.scene.layout.BorderPane;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import javafx.stage.Stage;
import javafx.stage.Window;
import netscape.javascript.JSObject;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.*;
import org.apache.http.entity.BasicHttpEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import java.io.*;
import java.net.*;
import java.nio.charset.StandardCharsets;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

public class GDPRDialog extends JFedBrowser {
    private static final Logger LOG = LoggerFactory.getLogger(GDPRDialog.class);

    private Testbed testbed;

    protected boolean tcApproved = false;
    protected boolean tcDeclined = false;

    public static class GDPRDialogJavaScriptBridge extends JavaScriptBridge {
        @Nonnull private final GDPRDialog gdprDialog;
        public GDPRDialogJavaScriptBridge(@Nonnull GDPRDialog gdprDialog) {
            super(gdprDialog);
            this.gdprDialog = gdprDialog;
        }

        public void approve() {
            LOG.debug("GDPRDialogJavaScriptBridge.approve");
//            gdprDialog.centerBox.setStyle("-fx-background-color: lawngreen");
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: lawngreen");
            gdprDialog.tcApproved = true;
            gdprDialog.tcDeclined = false;
        }
        public void decline() {
            LOG.debug("GDPRDialogJavaScriptBridge.decline");
//            gdprDialog.centerBox.setStyle("-fx-background-color: orangered");
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: orangered");
            gdprDialog.tcApproved = false;
            gdprDialog.tcDeclined = true;
        }
        public void close() {
            LOG.debug("GDPRDialogJavaScriptBridge.close");
            Window stage = gdprDialog.scene.getWindow();
            stage.hide();
        }
    }

    @Inject
    public GDPRDialog(@Nonnull GeniUserProvider geniUserProvider,
                      @Nonnull SfaConnectionPool connectionPool,
                      @Nonnull JFedTrustStore trustStore,
                      @Nonnull JFedPreferences preferences) {
        super(geniUserProvider, connectionPool, trustStore, preferences);
    }

    public boolean hasApproved() {
        //fallback: assume user has approved
        return tcApproved || !tcDeclined;
    }

    public void setTestbed(Testbed testbed) {
        this.testbed = testbed;

        loadPageWhenReady();
    }

    @FXML
    @Override
    protected void initialize() {
        webEngine = reservationWebView.getEngine();

        URLStreamFactoryCustomizer.useDedicatedProxyForWebkit(
                new MyHttpUrlStreamHandler(), "https");
//        webEngine.load("https://www.wilab2.ilabt.iminds.be:12369/reservation/sfareservation.php3");
//        webEngine.load("https://jfed.ilabt.imec.be");
        initialized = true;

        webEngine.getLoadWorker().stateProperty().addListener(
                (ObservableValue<? extends Worker.State> ov, Worker.State oldState,
                 Worker.State newState) -> {
                    if (newState == Worker.State.SUCCEEDED) {
                        jsBridgeBackoff = 1;
                        registerJsBridge();
                    }
                });

        WebConsoleListener.setDefaultListener((webView, message, lineNumber, sourceId) -> {
            LOG.debug("[Webengine message] "+message + "[at " + lineNumber + "]");
        });

        loadPageWhenReady();
    }

    @Override
    protected void loadPageWhenReady() {
        if (!initialized)
            return;

        if (this.testbed == null)
            return;

        Platform.runLater(this::registerJsBridge);
        loadPage();
    }

    @Override
    protected Object makeJavascriptBridge() {
        return new GDPRDialogJavaScriptBridge(this);
    }

    protected void loadPage() {
        if (this.testbed.getGdprInfo() == null) {
            LOG.warn("Cannot show Terms & Conditions page for Testbed without gdprInfo");
            return;
        }

        String gdprUrl = this.testbed.getGdprInfo().getGrpdUrl();
        if (gdprUrl == null) {
            LOG.warn("Cannot show Terms & Conditions page for Testbed without gdprInfo.grpdUrl");
            return;
        }

        headerLabel.setText("Terms & Conditions for "+testbed.getLongName());

        //gdprUrl = "https://www.wall2.ilabt.iminds.be:12369/gdpr/login.php";
        webEngine.load(gdprUrl);
    }

}
